FFmpeg
Loading...
Searching...
No Matches
aacenc_utils.h
Go to the documentation of this file.
1/*
2 * AAC encoder utilities
3 * Copyright (C) 2015 Rostislav Pehlivanov
4 *
5 * This file is part of FFmpeg.
6 *
7 * FFmpeg is free software; you can redistribute it and/or
8 * modify it under the terms of the GNU Lesser General Public
9 * License as published by the Free Software Foundation; either
10 * version 2.1 of the License, or (at your option) any later version.
11 *
12 * FFmpeg is distributed in the hope that it will be useful,
13 * but WITHOUT ANY WARRANTY; without even the implied warranty of
14 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
15 * Lesser General Public License for more details.
16 *
17 * You should have received a copy of the GNU Lesser General Public
18 * License along with FFmpeg; if not, write to the Free Software
19 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
20 */
21
22/**
23 * @file
24 * AAC encoder utilities
25 * @author Rostislav Pehlivanov ( atomnuker gmail com )
26 */
27
28#ifndef AVCODEC_AACENC_UTILS_H
29#define AVCODEC_AACENC_UTILS_H
30
31#include "libavutil/ffmath.h"
32#include "aacenc.h"
33#include "aacenctab.h"
34#include "aactab.h"
35
36#define ROUND_STANDARD 0.4054f
37#define ROUND_TO_ZERO 0.1054f
38#define C_QUANT 0.4054f
39
40static inline float pos_pow34(float a)
41{
42 return sqrtf(a * sqrtf(a));
43}
44
45/**
46 * Quantize one coefficient.
47 * @return absolute value of the quantized coefficient
48 * @see 3GPP TS26.403 5.6.2 "Scalefactor determination"
49 */
50static inline int quant(float coef, const float Q, const float rounding)
51{
52 float a = coef * Q;
53 return sqrtf(a * sqrtf(a)) + rounding;
54}
55
56static inline float find_max_val(int group_len, int swb_size, const float *scaled)
57{
58 float maxval = 0.0f;
59 int w2, i;
60 for (w2 = 0; w2 < group_len; w2++) {
61 for (i = 0; i < swb_size; i++) {
62 maxval = FFMAX(maxval, scaled[w2*128+i]);
63 }
64 }
65 return maxval;
66}
67
68static inline int find_min_book(float maxval, int sf)
69{
71 int qmaxval, cb;
72 qmaxval = maxval * Q34 + C_QUANT;
73 if (qmaxval >= (FF_ARRAY_ELEMS(aac_maxval_cb)))
74 cb = 11;
75 else
76 cb = aac_maxval_cb[qmaxval];
77 return cb;
78}
79
80static inline float find_form_factor(int group_len, int swb_size, float thresh,
81 const float *scaled, float nzslope) {
82 const float iswb_size = 1.0f / swb_size;
83 const float iswb_sizem1 = 1.0f / (swb_size - 1);
84 const float ethresh = thresh;
85 float form = 0.0f, weight = 0.0f;
86 int w2, i;
87 for (w2 = 0; w2 < group_len; w2++) {
88 float e = 0.0f, e2 = 0.0f, var = 0.0f, maxval = 0.0f;
89 float nzl = 0;
90 for (i = 0; i < swb_size; i++) {
91 float s = fabsf(scaled[w2*128+i]);
92 maxval = FFMAX(maxval, s);
93 e += s;
94 e2 += s *= s;
95 /* We really don't want a hard non-zero-line count, since
96 * even below-threshold lines do add up towards band spectral power.
97 * So, fall steeply towards zero, but smoothly
98 */
99 if (s >= ethresh) {
100 nzl += 1.0f;
101 } else {
102 if (nzslope == 2.f)
103 nzl += (s / ethresh) * (s / ethresh);
104 else
105 nzl += ff_fast_powf(s / ethresh, nzslope);
106 }
107 }
108 if (e2 > thresh) {
109 float frm;
110 e *= iswb_size;
111
112 /** compute variance */
113 for (i = 0; i < swb_size; i++) {
114 float d = fabsf(scaled[w2*128+i]) - e;
115 var += d*d;
116 }
117 var = sqrtf(var * iswb_sizem1);
118
119 e2 *= iswb_size;
120 frm = e / FFMIN(e+4*var,maxval);
121 form += e2 * sqrtf(frm) / FFMAX(0.5f,nzl);
122 weight += e2;
123 }
124 }
125 if (weight > 0) {
126 return form / weight;
127 } else {
128 return 1.0f;
129 }
130}
131
132/** Return the minimum scalefactor where the quantized coef does not clip. */
133static inline uint8_t coef2minsf(float coef)
134{
135 return av_clip_uint8(log2f(coef)*4 - 69 + SCALE_ONE_POS - SCALE_DIV_512);
136}
137
138/** Return the maximum scalefactor where the quantized coef is not zero. */
139static inline uint8_t coef2maxsf(float coef)
140{
141 return av_clip_uint8(log2f(coef)*4 + 6 + SCALE_ONE_POS - SCALE_DIV_512);
142}
143
144/*
145 * Returns the closest possible index to an array of float values, given a value.
146 */
147static inline int quant_array_idx(const float val, const float *arr, const int num)
148{
149 int i, index = 0;
150 float quant_min_err = INFINITY;
151 for (i = 0; i < num; i++) {
152 float error = (val - arr[i])*(val - arr[i]);
153 if (error < quant_min_err) {
154 quant_min_err = error;
155 index = i;
156 }
157 }
158 return index;
159}
160
161/**
162 * approximates exp10f(-3.0f*(0.5f + 0.5f * cosf(FFMIN(b,15.5f) / 15.5f)))
163 */
164static av_always_inline float bval2bmax(float b)
165{
166 return 0.001f + 0.0035f * (b*b*b) / (15.5f*15.5f*15.5f);
167}
168
169/*
170 * Compute a nextband map to be used with SF delta constraint utilities.
171 * The nextband array should contain 128 elements, and positions that don't
172 * map to valid, nonzero bands of the form w*16+g (with w being the initial
173 * window of the window group, only) are left indetermined.
174 */
175static inline void ff_init_nextband_map(const SingleChannelElement *sce, uint8_t *nextband)
176{
177 unsigned char prevband = 0;
178 int w, g;
179 /** Just a safe default */
180 for (g = 0; g < 128; g++)
181 nextband[g] = g;
182
183 /** Now really navigate the nonzero band chain */
184 for (w = 0; w < sce->ics.num_windows; w += sce->ics.group_len[w]) {
185 for (g = 0; g < sce->ics.num_swb; g++) {
186 if (!sce->zeroes[w*16+g] && sce->band_type[w*16+g] < RESERVED_BT)
187 prevband = nextband[prevband] = w*16+g;
188 }
189 }
190 nextband[prevband] = prevband; /* terminate */
191}
192
193/*
194 * Updates nextband to reflect a removed band (equivalent to
195 * calling ff_init_nextband_map after marking a band as zero)
196 */
197static inline void ff_nextband_remove(uint8_t *nextband, int prevband, int band)
198{
199 nextband[prevband] = nextband[band];
200}
201
202/*
203 * Checks whether the specified band could be removed without inducing
204 * scalefactor delta that violates SF delta encoding constraints.
205 * prev_sf has to be the scalefactor of the previous nonzero, nonspecial
206 * band, in encoding order, or negative if there was no such band.
207 */
209 const uint8_t *nextband, int prev_sf, int band)
210{
211 return prev_sf >= 0
212 && sce->sf_idx[nextband[band]] >= (prev_sf - SCALE_MAX_DIFF)
213 && sce->sf_idx[nextband[band]] <= (prev_sf + SCALE_MAX_DIFF);
214}
215
216/*
217 * Checks whether the specified band's scalefactor could be replaced
218 * with another one without violating SF delta encoding constraints.
219 * prev_sf has to be the scalefactor of the previous nonzero, nonsepcial
220 * band, in encoding order, or negative if there was no such band.
221 */
222static inline int ff_sfdelta_can_replace(const SingleChannelElement *sce,
223 const uint8_t *nextband, int prev_sf, int new_sf, int band)
224{
225 return new_sf >= (prev_sf - SCALE_MAX_DIFF)
226 && new_sf <= (prev_sf + SCALE_MAX_DIFF)
227 && sce->sf_idx[nextband[band]] >= (new_sf - SCALE_MAX_DIFF)
228 && sce->sf_idx[nextband[band]] <= (new_sf + SCALE_MAX_DIFF);
229}
230
231/**
232 * linear congruential pseudorandom number generator
233 *
234 * @param previous_val pointer to the current state of the generator
235 *
236 * @return Returns a 32-bit pseudorandom integer
237 */
238static av_always_inline int lcg_random(unsigned previous_val)
239{
240 union { unsigned u; int s; } v = { previous_val * 1664525u + 1013904223 };
241 return v.s;
242}
243
244#define ERROR_IF(cond, ...) \
245 if (cond) { \
246 av_log(avctx, AV_LOG_ERROR, __VA_ARGS__); \
247 return AVERROR(EINVAL); \
248 }
249
250#define WARN_IF(cond, ...) \
251 if (cond) { \
252 av_log(avctx, AV_LOG_WARNING, __VA_ARGS__); \
253 }
254
255#endif /* AVCODEC_AACENC_UTILS_H */
#define SCALE_MAX_DIFF
maximum scalefactor difference allowed by standard
Definition aac.h:94
@ RESERVED_BT
Band types following are encoded differently from others.
Definition aac.h:74
#define SCALE_DIV_512
scalefactor difference that corresponds to scale difference in 512 times
Definition aac.h:91
#define SCALE_ONE_POS
scalefactor index that corresponds to scale=1.0
Definition aac.h:92
#define POW_SF2_ZERO
ff_aac_pow2sf_tab index corresponding to pow(2, 0);
Definition aac.h:97
static int quant_array_idx(const float val, const float *arr, const int num)
static void ff_init_nextband_map(const SingleChannelElement *sce, uint8_t *nextband)
static av_always_inline float bval2bmax(float b)
approximates exp10f(-3.0f*(0.5f + 0.5f * cosf(FFMIN(b,15.5f) / 15.5f)))
#define C_QUANT
static int find_min_book(float maxval, int sf)
static float find_max_val(int group_len, int swb_size, const float *scaled)
static float find_form_factor(int group_len, int swb_size, float thresh, const float *scaled, float nzslope)
static uint8_t coef2minsf(float coef)
Return the minimum scalefactor where the quantized coef does not clip.
static int ff_sfdelta_can_replace(const SingleChannelElement *sce, const uint8_t *nextband, int prev_sf, int new_sf, int band)
static int ff_sfdelta_can_remove_band(const SingleChannelElement *sce, const uint8_t *nextband, int prev_sf, int band)
static uint8_t coef2maxsf(float coef)
Return the maximum scalefactor where the quantized coef is not zero.
static void ff_nextband_remove(uint8_t *nextband, int prevband, int band)
static av_always_inline int lcg_random(unsigned previous_val)
linear congruential pseudorandom number generator
static float pos_pow34(float a)
AAC encoder data.
static const unsigned char aac_maxval_cb[]
Definition aacenctab.h:141
AAC data declarations.
float ff_aac_pow34sf_tab[428]
static double val(void *priv, double ch)
Definition aeval.c:77
#define i(width, name, range_min, range_max)
Definition cbs_h264.c:63
#define s(width, name)
Definition cbs_vp9.c:198
#define av_clip_uint8
Definition common.h:106
static __device__ float sqrtf(float a)
static __device__ float fabsf(float a)
internal math functions header
static av_always_inline float ff_fast_powf(float x, float y)
Compute x^y for floating point x, y.
Definition ffmath.h:62
int index
Definition gxfenc.c:90
int a
#define Q(q)
#define b
Definition input.c:43
#define u(width, name, range_min, range_max)
Definition cbs_apv.c:68
#define av_always_inline
Definition attributes.h:72
#define log2f(x)
Definition libm.h:411
uint8_t w
Definition llvidencdsp.c:39
#define FFMIN(a, b)
Definition macros.h:49
#define FFMAX(a, b)
Definition macros.h:47
#define INFINITY
const h264_weight_func weight
#define FF_ARRAY_ELEMS(a)
int num_swb
number of scalefactor window bands
Definition aacdec.h:178
uint8_t group_len[8]
Definition aacdec.h:175
Single Channel Element - used for both SCE and LFE elements.
Definition aacdec.h:217
uint8_t zeroes[128]
band is not coded
Definition aacenc.h:116
enum BandType band_type[128]
band types
Definition aacdec.h:221
IndividualChannelStream ics
Definition aacdec.h:218
int sf_idx[128]
scalefactor indices
Definition aacenc.h:115
static void error(const char *err)
const char * g
Definition vf_curves.c:128
static double cb(void *priv, double x, double y)
Definition vf_geq.c:247
static const uint8_t quant[64]
Definition vmixdec.c:71